@@ -11,7 +11,7 @@ BEGIN {
 
 BEGIN {
 	$Type::Tiny::AUTHORITY   = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION     = '1.011_001';
+	$Type::Tiny::VERSION     = '1.011_003';
 	$Type::Tiny::XS_VERSION  = '0.016';
 }
 
@@ -372,9 +372,13 @@ sub _dd
 		local $Data::Dumper::Terse    = 1;
 		local $Data::Dumper::Sortkeys = 1;
 		local $Data::Dumper::Maxdepth = 2;
-		my $str = Data::Dumper::Dumper($value);
-		$str = substr($str, 0, $N - 12).'...'.substr($str, -1, 1)
-			if length($str) >= $N;
+		my $str;
+		eval {
+			$str = Data::Dumper::Dumper($value);
+			$str = substr($str, 0, $N - 12).'...'.substr($str, -1, 1)
+				if length($str) >= $N;
+			1;
+		} or do { $str = 'which cannot be dumped' };
 		"Reference $str";
 	}
 }
@@ -2154,7 +2158,7 @@ Returns true if any of the list match the type.
 
 Returns true if all of the list match the type.
 
-  if ( Int->any(@numbers) ) {
+  if ( Int->all(@numbers) ) {
     say "they were all integers";
   }
 
