@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Utils::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Utils::VERSION   = '1.011_001';
+	$Type::Utils::VERSION   = '1.011_003';
 }
 
 $Type::Utils::VERSION =~ tr/_//d;
@@ -32,7 +32,13 @@ our @EXPORT_OK = (
 		dwim_type english_list
 		classifier
 	>,
+	"is",
 );
+our %EXPORT_TAGS = (
+	default => [ @EXPORT ],
+	all     => [ @EXPORT_OK ],
+);
+pop @{$EXPORT_TAGS{all}};  # remove 'is'
 
 require Exporter::Tiny;
 our @ISA = 'Exporter::Tiny';
@@ -614,6 +620,31 @@ sub dwim_type
 	$type;
 }
 
+my %is_cache;
+my %is_cache_coderef;
+sub is
+{
+	my ($type, $value) = @_;
+	my $caller = caller;
+	my $uniq = blessed($type) ? $type->{uniq} : $type;
+	if (!blessed $type) {
+		my $orig = $type;
+		$type = $is_cache{$caller}{$uniq} || eval { dwim_type($type, for => $caller) };
+		if (blessed $type) {
+			$is_cache{$caller}{$uniq} ||= $type;
+		}
+		else {
+			my $thing = Type::Tiny::_dd($orig);
+			substr($thing, 0, 1) = lc substr($thing, 0, 1);
+			require Carp;
+			Carp::carp("Expected type, but got $thing; returning false");
+			return undef;
+		}
+	}
+
+	0+!! ( $is_cache_coderef{$caller}{$uniq} ||= $type->compiled_check )->($value);
+}
+
 sub english_list
 {
 	my $conjunction = ref($_[0]) eq 'SCALAR' ? ${+shift} : 'and';
@@ -1073,6 +1104,23 @@ return undef.
 
 This function is not exported by default.
 
+=item C<< is($type, $value) >>
+
+Shortcut for C<< $type->check($value) >> but also if $type is a string,
+will look it up via C<dwim_type>.
+
+This function is not exported by default.
+This function is not even exported by C<< use Type::Utils -all >>.
+You must request it explicitly.
+
+  use Type::Utils "is";
+
+Beware using this in test scripts because it has the same name as a function
+exported by L<Test::More>. Note that you can rename this function if
+C<is> will cause conflicts:
+
+   use Type::Utils "is" => { -as => "isntnt" };
+
 =item C<< english_list(\$conjunction, @items) >>
 
 Joins the items with commas, placing a conjunction before the final item.
