@@ -4,12 +4,18 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '1.011_001';
+our $VERSION   = '1.011_003';
 
 $VERSION =~ tr/_//d;
 
 use Scalar::Util qw< blessed refaddr weaken >;
 
+BEGIN {
+	*__XS = eval { require Type::Tiny::XS; 'Type::Tiny::XS'->VERSION('0.022'); 1 }
+		? sub () { !!1 }
+		: sub () { !!0 };
+};
+
 our @EXPORT_OK = (
 	map(@{[ $_, "is_$_", "assert_$_" ]}, __PACKAGE__->type_names),
 	qw/to_TypeTiny/
@@ -139,52 +145,215 @@ sub _get_check_overload_sub {
 
 sub StringLike ()
 {
+	return $cache{StringLike} if $cache{StringLike};
 	require Type::Tiny;
-	$cache{StringLike} ||= "Type::Tiny"->new(
+	my %common = (
 		name       => "StringLike",
+		library    => __PACKAGE__,
 		constraint => sub {    defined($_   ) && !ref($_   ) or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[""])  },
 		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[""])/ },
-		library    => __PACKAGE__,
 	);
+	if ( __XS ) {
+		my $xsub     = Type::Tiny::XS::get_coderef_for('StringLike');
+		my $xsubname = Type::Tiny::XS::get_subname_for('StringLike');
+		$cache{StringLike} = "Type::Tiny"->new(
+			%common,
+			constraint => $xsub,
+			inlined    => sub {
+				$Type::Tiny::AvoidCallbacks
+					? qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[""])/
+					: qq/$xsubname($_[1])/
+			},
+		);
+	}
+	else {
+		$cache{StringLike} = "Type::Tiny"->new(%common);
+	}
 }
 
-sub HashLike ()
+sub HashLike (;@)
 {
+	return $cache{HashLike} if $cache{HashLike} && !@_;
 	require Type::Tiny;
-	$cache{HashLike} ||= "Type::Tiny"->new(
+	my %common = (
 		name       => "HashLike",
+		library    => __PACKAGE__,
 		constraint => sub {    ref($_   ) eq q[HASH] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[%{}])  },
 		inlined    => sub { qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\%{}])/ },
-		library    => __PACKAGE__,
+		constraint_generator => sub {
+			my $param = TypeTiny()->assert_coerce( shift );
+			my $check = $param->compiled_check;
+			sub {
+				my %hash = %$_;
+				for my $key (sort keys %hash) {
+					$check->($hash{$key}) or return 0;
+				}
+				return 1;
+			};
+		},
+		inline_generator => sub {
+			my $param = TypeTiny()->assert_coerce( shift );
+			return unless $param->can_be_inlined;
+			sub {
+				my $var  = pop;
+				my $code = sprintf(
+					'do { my $ok=1; my %%h = %%{%s}; for my $k (sort keys %%h) { ($ok=0,next) unless (%s) }; $ok }',
+					$var,
+					$param->inline_check('$h{$k}'),
+				);
+				return (undef, $code);
+			};
+		},
+		coercion_generator => sub {
+			my ($parent, $child, $param) = @_;
+			return unless $param->has_coercion;
+			my $coercible = $param->coercion->_source_type_union->compiled_check;
+			my $C = "Type::Coercion"->new(type_constraint => $child);
+			$C->add_type_coercions(
+				$parent => sub {
+					my $origref = @_ ? $_[0] : $_;
+					my %orig    = %$origref;
+					my %new;
+					for my $k (sort keys %orig) {
+						return $origref unless $coercible->($orig{$k});
+						$new{$k} = $param->coerce($orig{$k});
+					}
+					\%new;
+				},
+			);
+			return $C;
+		},
 	);
+	if ( __XS ) {
+		my $xsub     = Type::Tiny::XS::get_coderef_for('HashLike');
+		my $xsubname = Type::Tiny::XS::get_subname_for('HashLike');
+		$cache{HashLike} = "Type::Tiny"->new(
+			%common,
+			constraint => $xsub,
+			inlined    => sub {
+				$Type::Tiny::AvoidCallbacks
+					? qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\%{}])/
+					: qq/$xsubname($_[1])/
+			},
+		);
+	}
+	else {
+		$cache{HashLike} = "Type::Tiny"->new(%common);
+	}
+	
+	@_ ? $cache{HashLike}->parameterize(@{$_[0]}) : $cache{HashLike};
 }
 
-sub ArrayLike ()
+sub ArrayLike (;@)
 {
+	return $cache{ArrayLike} if $cache{ArrayLike} && !@_;
 	require Type::Tiny;
-	$cache{ArrayLike} ||= "Type::Tiny"->new(
+	my %common = (
 		name       => "ArrayLike",
+		library    => __PACKAGE__,
 		constraint => sub {    ref($_   ) eq q[ARRAY] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[@{}])  },
 		inlined    => sub { qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\@{}])/ },
-		library    => __PACKAGE__,
+		constraint_generator => sub {
+			my $param = TypeTiny()->assert_coerce( shift );
+			my $check = $param->compiled_check;
+			sub {
+				my @arr = @$_;
+				for my $val (@arr) {
+					$check->($val) or return 0;
+				}
+				return 1;
+			};
+		},
+		inline_generator => sub {
+			my $param = TypeTiny()->assert_coerce( shift );
+			return unless $param->can_be_inlined;
+			sub {
+				my $var  = pop;
+				my $code = sprintf(
+					'do { my $ok=1; for my $v (@{%s}) { ($ok=0,next) unless (%s) }; $ok }',
+					$var,
+					$param->inline_check('$v'),
+				);
+				return (undef, $code);
+			};
+		},
+		coercion_generator => sub {
+			my ($parent, $child, $param) = @_;
+			return unless $param->has_coercion;
+			my $coercible = $param->coercion->_source_type_union->compiled_check;
+			my $C = "Type::Coercion"->new(type_constraint => $child);
+			$C->add_type_coercions(
+				$parent => sub {
+					my $origref = @_ ? $_[0] : $_;
+					my @orig    = @$origref;
+					my @new;
+					for my $v (@orig) {
+						return $origref unless $coercible->($v);
+						push @new, $param->coerce($v);
+					}
+					\@new;
+				},
+			);
+			return $C;
+		},
 	);
+	if ( __XS ) {
+		my $xsub     = Type::Tiny::XS::get_coderef_for('ArrayLike');
+		my $xsubname = Type::Tiny::XS::get_subname_for('ArrayLike');
+		$cache{ArrayLike} = "Type::Tiny"->new(
+			%common,
+			constraint => $xsub,
+			inlined    => sub {
+				$Type::Tiny::AvoidCallbacks
+					? qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\@{}])/
+					: qq/$xsubname($_[1])/
+			},
+		);
+	}
+	else {
+		$cache{ArrayLike} = "Type::Tiny"->new(%common);
+	}
+	
+	@_ ? $cache{ArrayLike}->parameterize(@{$_[0]}) : $cache{ArrayLike};
+}
+
+if ( $] ge '5.014' ) {
+	&Scalar::Util::set_prototype($_, ';$') for \&HashLike, \&ArrayLike;
 }
 
 sub CodeLike ()
 {
+	return $cache{CodeLike} if $cache{CodeLike};
 	require Type::Tiny;
-	$cache{CodeLike} ||= "Type::Tiny"->new(
+	my %common = (
 		name       => "CodeLike",
 		constraint => sub {    ref($_   ) eq q[CODE] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[&{}])  },
 		inlined    => sub { qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\&{}])/ },
 		library    => __PACKAGE__,
 	);
+	if ( __XS ) {
+		my $xsub     = Type::Tiny::XS::get_coderef_for('CodeLike');
+		my $xsubname = Type::Tiny::XS::get_subname_for('CodeLike');
+		$cache{CodeLike} = "Type::Tiny"->new(
+			%common,
+			constraint => $xsub,
+			inlined    => sub {
+				$Type::Tiny::AvoidCallbacks
+					? qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\&{}])/
+					: qq/$xsubname($_[1])/
+			},
+		);
+	}
+	else {
+		$cache{CodeLike} = "Type::Tiny"->new(%common);
+	}
 }
 
 sub TypeTiny ()
 {
+	return $cache{TypeTiny} if $cache{TypeTiny};
 	require Type::Tiny;
-	$cache{TypeTiny} ||= "Type::Tiny"->new(
+	my %common = (
 		name       => "TypeTiny",
 		constraint => sub {  Scalar::Util::blessed($_   ) && $_   ->isa(q[Type::Tiny])  },
 		inlined    => sub { my $var = $_[1]; "Scalar::Util::blessed($var) && $var\->isa(q[Type::Tiny])" },
@@ -195,12 +364,30 @@ sub TypeTiny ()
 			$c->freeze;
 		},
 	);
+	if ( 0 and __XS ) {  # causes fails in 03-leak.t
+		my $xsub     = Type::Tiny::XS::get_coderef_for('InstanceOf[Type::Tiny]');
+		my $xsubname = Type::Tiny::XS::get_subname_for('InstanceOf[Type::Tiny]');
+		$cache{TypeTiny} = "Type::Tiny"->new(
+			%common,
+			constraint => $xsub,
+			inlined    => sub {
+				my $var = $_[1];
+				$Type::Tiny::AvoidCallbacks
+					? "Scalar::Util::blessed($var) && $var\->isa(q[Type::Tiny])"
+					: qq/$xsubname($var)/
+			},
+		);
+	}
+	else {
+		$cache{TypeTiny} = "Type::Tiny"->new(%common);
+	}
 }
 
 sub _ForeignTypeConstraint ()
 {
+	return $cache{_ForeignTypeConstraint} if $cache{_ForeignTypeConstraint};
 	require Type::Tiny;
-	$cache{_ForeignTypeConstraint} ||= "Type::Tiny"->new(
+	$cache{_ForeignTypeConstraint} = "Type::Tiny"->new(
 		name       => "_ForeignTypeConstraint",
 		constraint => \&_is_ForeignTypeConstraint,
 		inlined    => sub { qq/ref($_[1]) && do { require Types::TypeTiny; Types::TypeTiny::_is_ForeignTypeConstraint($_[1]) }/ },
